@@ -4,7 +4,7 @@ import sys, re, os
 
 help = """
 Usage:
-    log_check [option] [paths|files]
+    logc [option] [paths|files]
 
 Option:
     -S, -show_ok                show ok file.
@@ -12,9 +12,9 @@ Option:
     -h, -help                   just show this.
 
 Example:
-    log_ckeck                   check all .cpp, .h, .hpp, .mm in `./`
-    log_check ../crcp           check all .cpp, .h, .hpp, .mm in `../crcp`
-    log_check main.cpp          check main.cpp
+    logc                        check all .cpp, .h, .hpp, .mm in `./`
+    logc ../crcp                check all .cpp, .h, .hpp, .mm in `../crcp`
+    logc main.cpp               check main.cpp
 """
 
 g_unknow_logs = []
@@ -38,7 +38,10 @@ class Progress:
 
 
 def getfiles(current_dir, types, files = []):
-    dir_list = os.listdir(current_dir)
+    try:
+        dir_list = os.listdir(current_dir)
+    except PermissionError:
+        return None
     for dir in dir_list:
         path = os.path.join(current_dir, dir)
         if os.path.isdir(path):
@@ -51,15 +54,28 @@ def getfiles(current_dir, types, files = []):
     return files
 
 
-# return end_pos and arg_num
 def parse(text, start_pos):
+    first_quote_idx = text.find('\"', start_pos)
+    in_dou_quote = False
+    fmt_end_pos = None
+    for i in range(first_quote_idx, len(text)):
+        c = text[i] 
+        if c == '\"':
+            in_dou_quote = not in_dou_quote
+        if not in_dou_quote and (c==',' or c==')'):
+            fmt_end_pos = i
+            break
+    if fmt_end_pos == None:
+        raise Exception('fmt_end_pos is None')
+    place_hoder_num = len( re.findall(r'({.*?})', text[start_pos:fmt_end_pos]) )
+
+    i = fmt_end_pos
+    arg_num = 0
     left_braces = {'(', '[', '{'}
     right_braces = {')', ']', '}'}
-    deep = 1
-    i = start_pos
-    arg_num = 0
     in_dou_quote = False
     in_sin_quote = False
+    deep = 1
     while deep!=0 and i<len(text):
         c = text[i]
         if c == '"':
@@ -74,35 +90,21 @@ def parse(text, start_pos):
         if c == ',' and deep == 1 and not in_dou_quote and not in_sin_quote:
             arg_num = arg_num+1
         i = i+1
-    end_pos = i+1
+    end_pos = i
     if deep != 0:
         raise Exception('deep({}) error'.format(deep))
-    if text[end_pos-1] != ';':
-        raise Exception('parse error, not end with;')
-    return (end_pos, arg_num)
-
-def count_num_brace(per_log):
-    first_quote_idx = per_log.find('\"')
-    second_quote_idx = per_log.find('\"', first_quote_idx+1)
-    if first_quote_idx == -1 or second_quote_idx == -1:
-        raise Exception('num of `\"` < 2 : first_quote_idx = {}, second_quote_idx = {}'.format(first_quote_idx, second_quote_idx))
-
-    format_str = per_log[first_quote_idx+1:second_quote_idx]
-    res = re.findall(r'({.*?})', format_str)
-    if res:
-        return len(res)
-    return 0
 
+    statement = text[start_pos:end_pos]
+    return ((place_hoder_num, arg_num, statement))
 
 bad_log_fmt = """
-
-
-{}
---------------------------------------------------------------
 {} [line {}] : {{}} = {}, argc = {}
---------------------------------------------------------------"""
+{}
+"""
 
 def check(files, show_ok):
+    if len(files) == 0:
+        return
     max_file_name_len = max([len(file_name) for file_name in files])
 
     bad_files = []
@@ -118,47 +120,43 @@ def check(files, show_ok):
         except UnicodeDecodeError:
             print('\n can not read file `{}`'.format(source_file))
             continue
-        iter = re.finditer('(LOG[T|D|I|W|E|C]\(.*?\n{0,}.*?\);)', text, re.MULTILINE)
+        iter = re.finditer('(LOG[T|D|I|W|E|C]\(|fmt::format\()', text, re.MULTILINE)
 
-        bad_logs = []
+        bad_fmt = []
         log_len = 0
         for item in iter:
             log_len = log_len+1
-            per_log = item.group()
+            key_word = item.group()
             location = len(re.findall('\n', text[:item.start()])) + 1
             try:
-                end_pos, num_arg = parse(text, item.start() + per_log.index(',')+1)
-                if end_pos != item.end():
-                    per_log = text[item.start():end_pos]
-                
-                num_brace = count_num_brace(per_log)
+                place_hoder_num, arg_num, statement = parse(text, item.start())
             except Exception as e:
-                g_unknow_logs.append({'file':source_file, 'line':location, 'exception': e, 'log':per_log})
+                g_unknow_logs.append({'file':source_file, 'line':location, 'exception': e, 'key_word':key_word})
                 continue
                 
-            if num_brace != num_arg:
-                print(bad_log_fmt.format(per_log, source_file, location, num_brace, num_arg))
-                bad_logs.append((per_log, num_brace, num_arg))
+            if place_hoder_num != arg_num:
+                print(bad_log_fmt.format(source_file, location, place_hoder_num, arg_num, statement))
+                bad_fmt.append((key_word, place_hoder_num, arg_num))
             
         space_num = max_file_name_len-len(source_file)
-        if len(bad_logs) == 0:
+        if len(bad_fmt) == 0:
             if show_ok:
                 space_num = max_file_name_len-len(source_file)
-                print('{}{} all {:2} LOGs ok'.format(source_file, ' '*space_num, log_len))
+                print('{}{} all {:2} `Fmt` ok'.format(source_file, ' '*space_num, log_len))
         else:
-            print('{}{} {:2} bad LOGs total\n'.format(source_file, ' '*space_num, len(bad_logs)))
-            bad_files.append({'source_file':source_file, 'bad_logs': bad_logs})
+            print('{}{} {:2} bad `Fmt` total in {} `Fmt`\n'.format(source_file, ' '*space_num, len(bad_fmt), log_len))
+            bad_files.append({'source_file':source_file, 'bad_fmt': bad_fmt})
         
         if not show_ok:
             progress.next()
 
     print()
     print()
-    print('({}/{})(badfile/total) {:3} bad LOG total'.format(len(bad_files), len(files), sum([len(bad_file['bad_logs']) for bad_file in bad_files])))
+    print('({}/{})(badfile/total) {:3} bad `Fmt` total'.format(len(bad_files), len(files), sum([len(bad_file['bad_fmt']) for bad_file in bad_files])))
     print()
     print()
     print()
-    print('{} unknown LOGs:'.format(len(g_unknow_logs)))
+    print('{} unknown `Fmt`:'.format(len(g_unknow_logs)))
     for log in g_unknow_logs:
         print(log)
 
@@ -189,7 +187,7 @@ def main():
                 files.append(arg)
                 print('{} is file'.format(arg))
             else:
-                print('`{}` is not file or dir!!!', arg)
+                print('`{}` is not file or dir!!!'.format(arg))
         check(files, show_ok)
     else:
         check(getfiles('./', default_types), show_ok)
